import string
import six
import logging
import re

from ipalib import api, errors, output, _
from ipalib import Str, StrEnum, Bool, Int, _
from ipalib.plugable import Registry
from . import baseldap
from .baseldap import (LDAPObject, LDAPCreate, LDAPDelete,
                                     LDAPUpdate, LDAPSearch, LDAPRetrieve,
                                     LDAPQuery, LDAPAddMember, LDAPRemoveMember,
                                     add_external_pre_callback,
                                     add_external_post_callback,
                                     remove_external_post_callback,
                                     output, entry_to_dict, pkey_to_value,
                                     LDAPAddAttribute, LDAPRemoveAttribute,
                                     LDAPModAttribute)
from .hbacrule import is_all
from ipalib import _, ngettext
from ipalib.util import validate_hostmask
from ipapython.dn import DN

if six.PY3:
    unicode = str


__doc__ = _("""
Parsec registered devices

Manage Parsec registered devices.

""")

register = Registry()

#topic = 'parsecdevice'

@register()
class parsecdevice(baseldap.LDAPObject):
    container_dn = DN(('cn', 'devices'),
                      ('cn', 'dev-services'),
                      ('cn', 'parsec-configs'),
                      ('cn', 'etc'))
    object_name = _('parsecdevice')
    object_name_plural = _('parsecdevices')
    object_class = ['top', 'nscontainer', 'x-ald-device']
    default_attributes = [
        'cn',
        'description',
        'x-ald-devac-attr',
        'x-ald-devac-owner',
        'x-ald-devac-group',
        'x-ald-devac-status',
        'x-ald-devac-mac',
        'x-ald-devac-mac-type',
        'x-ald-devac-mode',
        'x-ald-devac-aud',
        'x-ald-mac-level',
        'x-ald-mac-category',
        'x-ald-devac-rule',
        'audmasksucc',
        'audmaskfail',
        'memberof'
    ]
    attribute_members = {
        'memberof': [ 'user', 'group', 'macdevcat', 'audmasksucc', 'audmaskfail', 'parsecrule' ],
        'memberofindirect': ['group', 'netgroup', 'role', 'hbacrule', 'sudorule', 'macdevcat', 'audmasksucc', 'audmaskfail' ],
    }
    label = _('Registered Devices')
    label_singular = _('Registered Device')

    takes_params = (
        Str('cn',
            cli_name='device_name',
            label=_('Name of registered device'),
            primary_key=True,
        ),
        Str('description?',
            cli_name='device_description',
            label=_('Device description'),
        ),
        Bool('x-ald-devac-status?',
            cli_name='device_status',
            label=_('Device is ON:'),
        ),
        Str('x-ald-devac-owner',
            cli_name='device_owner',
#            multivalue=True,
            label=_('Device owner'),
        ),
        Str('x-ald-devac-group',
            cli_name='device_group',
#            multivalue=True,
            label=_('Device group'),
        ),
        Str('x-ald-devac-attr',
            cli_name='device_attr',
            multivalue=True,
            label=_('Device attr'),
        ),
        Str('x-ald-devac-rule?',
            cli_name='device_rule',
            multivalue=True,
            label=_('Device rule'),
        ),
        Str('x-ald-devac-mode?',
            cli_name='device_mode',
            label=_('Device mode'),
        ),
#        Str('x-ald-devac-mac?',
#            cli_name='device_mac',
#            label=_('Device MAC'),
#        ),
#        Str('x-ald-devac-aud?',
#            cli_name='device_aud',
#            label=_('Device audit'),
#        ),
        Str('x-ald-mac-level?',
            cli_name='device_lev',
            label=_('Device level'),
        ),
        Str('x-ald-mac-category?',
            cli_name='device_cat',
            label=_('Device cat'),
        ),
        Str('audmasksucc?',
            cli_name='device_audsucc',
            label=_('Device audsucc'),
        ),
        Str('audmaskfail?',
            cli_name='device_audfail',
            label=_('Device audfail'),
        ),
#        Str('x-ald-devac-mac-type?',
#           cli_name='device_type',
#           label=_('Device type'),
#       ),
    )

@register()
class parsecdevice_find(LDAPSearch):
    __doc__ = _('Search for registered devices.')

    msg_summary = ngettext(
        '%(count)d registered device matched',
        '%(count)d registered device matched', 0
    )
    def post_callback(self, ldap, entries, truncated, *args, **options):
        if entries:
            mode = entries[-1].get('x-ald-devac-mode','0')
            entries[-1]['x-ald-devac-mode'] = [ format(int(mode[-1]), '03o') ]
        return truncated

@register()
class parsecdevice_show(LDAPRetrieve):
    __doc__ = _('Display registered device.')

    msg_summary = "Device %(value)s"
    has_output = (
        output.summary,
        output.Entry('result'),
        output.value,
    )
    def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
        mode = entry_attrs.get('x-ald-devac-mode','0')
        entry_attrs['x-ald-devac-mode'] = [ format(int(mode[-1]), '03o') ]
        return dn


def format_devac_attr( attr_in):
    attr_out = []
    for attr in attr_in:
        logging.debug( 'PARSEC: attr=%s', attr)
        if not re.match( r'.*[=+]=.*', attr):
            if re.match( r'ID_SERIAL=.*', attr):
                attr = 'ENV{ID_SERIAL}=="' + attr[10:] + '"'
            else:
                if re.match(r'ID_SERIAL_SHORT=.*', attr):
                    attr = 'ENV{ID_SERIAL_SHORT}=="' + attr[16:] + '"'
                else:
                    attr = attr.replace( '=', '==', 1)
        attr_out.append( attr )
        logging.debug( 'PARSEC: attr_out=%s', attr_out)
    return attr_out


@register()
class parsecdevice_mod(LDAPUpdate):
    __doc__ = _('Modify registered device.')

    msg_summary = _('Modified registered device"%(value)s"')

    def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):

        if not ldap.get_entry(dn):
            entry_attrs['x-ald-devac-mac'] = '0:0x0'
            entry_attrs['x-ald-devac-aud'] = '0x0:0x0'
        else:
            logging.debug( 'PARSEC: parsecdevice_mod keys=%s', keys)
            logging.debug( 'PARSEC: parsecdevice_mod entry_attrs=%s', entry_attrs)
            logging.debug( 'PARSEC: parsecdevice_mod entry_list=%s', attrs_list)
            logging.debug( 'PARSEC: parsecdevice_mod options=%s', options)

            old_entry = ldap.get_entry(dn, ['x-ald-mac-level', 'x-ald-mac-category', 'audmasksucc', 'audmaskfail', 'x-ald-devac-rule'])
            logging.debug( 'PARSEC: parsecdevice_mod old_entry=%s', old_entry)

            if entry_attrs.get('x-ald-devac-mode') is not None:
                mode = entry_attrs.get('x-ald-devac-mode')
                if re.match( r'[r-][w-][x-][r-][w-][x-][r-][w-][x-]$', mode):
                    out_mode = 0
                    if mode[0]=='r':
                        out_mode += 0o400
                    if mode[1]=='w':
                        out_mode += 0o200
                    if mode[2]=='x':
                        out_mode += 0o100
                    if mode[3]=='r':
                        out_mode += 0o40
                    if mode[4]=='w':
                        out_mode += 0o20
                    if mode[5]=='x':
                        out_mode += 0o10
                    if mode[6]=='r':
                        out_mode += 0o4
                    if mode[7]=='w':
                        out_mode += 0o2
                    if mode[8]=='x':
                        out_mode += 0o1
                else:
                    if re.match( r'[0-7][0-7][0-7]$', mode):
                        out_mode = int( mode, 8)
                    else:
                        raise errors.ValidationError(message=_("Wrong device access mode specified"))

                entry_attrs['x-ald-devac-mode'] = out_mode

            if entry_attrs.get('x-ald-devac-attr') is not None:
                attr_in = entry_attrs.get('x-ald-devac-attr')
                entry_attrs['x-ald-devac-attr'] = format_devac_attr( attr_in)

            if entry_attrs.get('x-ald-mac-level') is None:
                lev = old_entry['x-ald-mac-level'][0]
            else:
                lev = entry_attrs.get('x-ald-mac-level')

            if entry_attrs.get('x-ald-mac-category') is None:
                cat = old_entry['x-ald-mac-category'][0]
            else:
                cat = entry_attrs.get('x-ald-mac-category')

            if entry_attrs.get('audmasksucc') is None:
                succ = old_entry['audmasksucc'][0]
            else:
                succ = entry_attrs.get('audmasksucc')

            if entry_attrs.get('audmaskfail') is None:
                fail = old_entry['audmaskfail'][0]
            else:
                fail = entry_attrs.get('audmaskfail')

            mac = '%s:%s' % (lev, hex(int(cat, 16)))
            logging.debug( 'PARSEC: mac=%s', mac)
            entry_attrs['x-ald-devac-mac'] = mac

            aud = '%s:%s' % (hex(int(succ, 16)), hex(int(fail, 16)))
            logging.debug( 'PARSEC: aud=%s', aud)
            entry_attrs['x-ald-devac-aud'] = aud

        return dn

    # BT-37159
    def post_callback(self, ldap, dn, entry_attrs, *keys, **options): 
        mode = entry_attrs.get('x-ald-devac-mode','0')
        entry_attrs['x-ald-devac-mode'] = [ format(int(mode[-1]), '03o') ]
        return dn

@register()
class parsecdevice_add(LDAPCreate):
    __doc__ = _('Create new registered device.')

    msg_summary = _('Added registered device "%(value)s"')

    def xinit( self, entry_attrs, entry_name, entry_value):
        try:
            attr_in = entry_attrs.get(entry_name)
        except KeyError:
            entry_attrs[entry_name] = entry_value
        else:
            if attr_in is None:
                entry_attrs[entry_name] = entry_value
            else:
                entry_attrs[entry_name] = attr_in
                if entry_name == 'x-ald-devac-mode': # BT-37159
                    entry_attrs['x-ald-devac-mode'] = int(entry_attrs['x-ald-devac-mode'], 8) 

    def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):

#        entry_attrs['uid'] = entry_attrs.get('cn')
        self.xinit( entry_attrs, 'x-ald-mac-level', 0)
        self.xinit( entry_attrs, 'x-ald-mac-category', '0')
        self.xinit( entry_attrs, 'x-ald-devac-mac', '0:0x0')
        self.xinit( entry_attrs, 'audmaskfail', '0')
        self.xinit( entry_attrs, 'audmasksucc', '0')
        self.xinit( entry_attrs, 'x-ald-devac-aud', '0x0:0x0')
        self.xinit( entry_attrs, 'x-ald-devac-status', False)
        self.xinit( entry_attrs, 'x-ald-devac-mode', int( "640", 8))
        try:
            attr_in = entry_attrs.get('x-ald-devac-attr')
        except KeyError:
            pass
        else:
            if entry_attrs.get('x-ald-devac-attr') is not None:
                entry_attrs['x-ald-devac-attr'] = format_devac_attr( attr_in)
        return dn
    # BT-37159
    def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
        mode = entry_attrs.get('x-ald-devac-mode','0')
        entry_attrs['x-ald-devac-mode'] = [ format(int(mode[-1]), '03o') ]
        return dn

@register()
class parsecdevice_del(LDAPDelete):
    __doc__ = _('Delete registered device.')

    msg_summary = _('Deleted registered device"%(value)s"')
